open Core

type t = {
  user_id: String.t;
  date: Date.t;
  content: String.t;
  created_at: Time.t;
  updated_at: Time.t;
} [@@deriving equal, sexp]

let of_sql_row = function[@warning "-4"]
  | [| "user_id", Sqlite3.Data.TEXT user_id;
       "date", Sqlite3.Data.TEXT date_str;
       "content", Sqlite3.Data.TEXT content;
       "created_at", Sqlite3.Data.TEXT created_at_str;
       "updated_at", Sqlite3.Data.TEXT updated_at_str;
    |] ->
    let date = Core.Date.of_string_iso8601_basic ~pos:0 date_str in
    let created_at = Core.Time.of_string created_at_str in
    let updated_at = Core.Time.of_string updated_at_str in
    { user_id; date; content; created_at; updated_at }
  | row -> raise (Oksql.Conversion_failed row)

let to_string entry =
  Sexp.to_string (sexp_of_t entry)
